package game

import (
  "fmt"
  "given_files/player"
  "os"
  "strconv"
)

type Move int64

const (
  UP Move = iota
  DOWN
  LEFT
  RIGHT
)

type Map struct {
  Height, Width, timeout int
  Walls [][]bool
  Self player.Player
  Others []player.Player
}

func Init() *Map {
  var input string
  fmt.Scan(&input)
  this := new(Map)
  t, _ := strconv.Atoi(input)
  this.timeout = t
  fmt.Scan(&input)
  w, _ := strconv.Atoi(input)
  fmt.Scan(&input)
  h, _ := strconv.Atoi(input)
  this.Height = h
  this.Width = w
  this.Walls = make([][]bool, this.Height)
  for i := 0; i < this.Height; i++ { 
    this.Walls[i] = make([]bool, this.Width)
  }
  for i := 0; i < this.Height; i++ {
    for j := 0; j < this.Width; j++ {
      this.Walls[i][j] = false
    }
  }
  this.Self = player.NewPlayerEntry()
  this.Others = make([]player.Player, 0)
  return this
}

func (this *Map) Clone() (res Map) {
  i := make([][]bool, this.Height)
  for a := 0; a < this.Height; a++ {
    tmp := make([]bool, this.Width)
    copy(tmp, this.Walls[a])
    i[a] = tmp
  }
  j := make([]player.Player, len(this.Others))
  for k, p := range this.Others {
    j[k] = p.Clone()
  }
  res = Map {this.Height, this.Width, this.timeout, i, this.Self.Clone(), j}
  return
}

func (this *Map) Update() {
  var input string
  fmt.Scan(&input)
  playerCount, err := strconv.Atoi(input)
  if err != nil {
    os.Exit(0)
  }
  if playerCount == 0 {
    return
  }
  this.Others = make([]player.Player, playerCount)
  tmp := make([]player.Player, playerCount)
  p1, p2 := 0, 0
  passedSelf := false

  for i := 0; i < playerCount; i++ {
    p := player.NewPlayerEntry()
    this.SetWall(p.X, p.Y, true)
    if p.X != this.Self.X || p.Y != this.Self.Y {
      if passedSelf {
	this.Others[p1] = p
	p1 = p1 + 1
      } else {
	tmp[p2] = p
	p2 = p2 + 1
      }
    } else {
      passedSelf = true
    }
  }
  for i := 0; i < p2; i++ {
    this.Others[p1] = tmp[i]
    p1 = p1 + 1
  }
}

func (this *Map) IsWall(X int, Y int) bool {
  if X < 0 || X >= this.Width || Y < 0 || Y >= this.Height {
    return true
  }
  return this.Walls[Y][X]
}

func (this *Map) SetWall(X int, Y int, b bool) bool {
  if X < 0 || X > this.Width || Y < 0 || Y > this.Height {
    return false
  }
  this.Walls[Y][X] = b
  return true
}

func (this *Map) Move(move Move) {
  this.SetWall(this.Self.X, this.Self.Y, true) 
  switch move {
    case UP:
      this.Self.Y--
      fmt.Println("U")
    case DOWN:
      this.Self.Y++
      fmt.Println("D")
    case LEFT:
      this.Self.X--
      fmt.Println("L")
    case RIGHT:
      this.Self.X++
      fmt.Println("R")
  }
}

func MoveTo(m Map, act Move, this *player.Player) bool {
  newX, newY := this.X, this.Y
  switch (act) {
    case UP:
      newY--
    case DOWN:
      newY++
    case LEFT:
      newX--
    case RIGHT:
      newX++
  }
  if newX >= 0 && newX < m.Width &&
      newY >= 0 && newY < m.Height && 
      !m.IsWall(newX, newY) {
    this.X, this.Y = newX, newY
    m.SetWall(newX, newY, true)
    return true
  }
  return false
}

